//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: StatsLookupCommand.cpp,v 1.2 2004/09/06 14:01:14 jsnelson Exp $
//

#include "StatsLookupCommand.hpp"

using namespace DbXml;
using namespace std;

string StatsLookupCommand::getCommandName() const
{
	return "statslookup";
}

string StatsLookupCommand::getBriefHelp() const
{
	return "Looks up index statistics on the default container";
}

string StatsLookupCommand::getMoreHelp() const
{
	return
		string("Takes arguments of the index descriptor string, the namespace and\n") +
		string("name of the element or attribute, and optionally the parent namespace\n") +
		string("and name, and the value to lookup . This command uses the\n") +
		string("XmlContainer::lookupStatistics() method.\n");
}

void StatsLookupCommand::execute(Args &args, Environment &env)
{
	if(args.size() < 4 || args.size() > 7 || args.size() == 5) {
		throw CommandException("Wrong number of arguments");
	}
	env.testContainer();

	XmlValue value;
	if(args.size() == 7) {
		value = args[6];
	}

	if(env.txn()) {
		if(args.size() == 4 || args[5] == "") {
			XmlStatistics stats = env.container()->
				lookupStatistics(*env.txn(),
						 args[2], args[3],
						 args[1], value);

			cout << "Number of Indexed Keys: " << stats.getNumberOfIndexedKeys() << endl;
			cout << "Number of Unique Keys:  " << stats.getNumberOfUniqueKeys() << endl;
			cout << "Sum Key Value Size:     " << stats.getSumKeyValueSize() << endl << endl;
		}
		else {
			XmlStatistics stats = env.container()->
				lookupStatistics(*env.txn(),
						 args[2], args[3],
						 args[4], args[5],
						 args[1], value);

			cout << "Number of Indexed Keys: " << stats.getNumberOfIndexedKeys() << endl;
			cout << "Number of Unique Keys:  " << stats.getNumberOfUniqueKeys() << endl;
			cout << "Sum Key Value Size:     " << stats.getSumKeyValueSize() << endl << endl;
		}
	}
	else {
		if(args.size() == 4 || args[5] == "") {
			XmlStatistics stats = env.container()->
				lookupStatistics(args[2], args[3],
						 args[1], value);

			cout << "Number of Indexed Keys: " << stats.getNumberOfIndexedKeys() << endl;
			cout << "Number of Unique Keys:  " << stats.getNumberOfUniqueKeys() << endl;
			cout << "Sum Key Value Size:     " << stats.getSumKeyValueSize() << endl << endl;
		}
		else {
			XmlStatistics stats = env.container()->
				lookupStatistics(args[2], args[3],
						 args[4], args[5],
						 args[1], value);

			cout << "Number of Indexed Keys: " << stats.getNumberOfIndexedKeys() << endl;
			cout << "Number of Unique Keys:  " << stats.getNumberOfUniqueKeys() << endl;
			cout << "Sum Key Value Size:     " << stats.getSumKeyValueSize() << endl << endl;
		}
	}
}

