//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: UpdateCommand.cpp,v 1.4 2004/10/18 22:52:13 gmf Exp $
//

#include "UpdateCommand.hpp"

using namespace DbXml;
using namespace std;

string UpdateCommand::getCommandName() const
{
  return "update";
}

string UpdateCommand::getBriefHelp() const
{
  return "Update node contents from the documents in the last results returned";
}

string UpdateCommand::getMoreHelp() const
{
  return
    string("Takes the path to the nodes to update, and the new content as\n") +
    string("arguments. The results of the modifications are left in the\n") +
    string("default results. This command creates an XmlModify object using\n") +
    string("XmlManager::createModify(), then adds an update step to it using\n") +
    string("XmlModify::addUpdateStep(). It then executes using\n") +
    string("XmlModify::execute().");
}

void UpdateCommand::execute(Args &args, Environment &env)
{
  if(args.size() != 3) {
    throw CommandException("Wrong number of arguments");
  }
  env.testResults();

  if(env.verbose()) cout << "Updating nodes: " << args[1] << " to have content: " << args[2] << endl;

  XmlModify modify = env.db().createModify();
  env.results()->reset();

  unsigned int modifications;
  if(env.txn()) {
	  XmlTransaction myTxn = env.childTransaction();
	  modify.addUpdateStep(env.db().prepare(myTxn, args[1], env.context()), args[2]);
	  modifications = modify.execute(myTxn, *env.results(), env.context(), env.uc());
	  myTxn.commit();
  }
  else {
	  modify.addUpdateStep(env.db().prepare(args[1], env.context()), args[2]);
	  modifications = modify.execute(*env.results(), env.context(), env.uc());
  }


  if(env.verbose()) cout << modifications << " modifications made." << endl;
}

